@@ -1,4 +1,4 @@
-#!/pro/bin/perl
+#!/usr/bin/perl
 
 package Spreadsheet::Read;
 
@@ -15,6 +15,7 @@ package Spreadsheet::Read;
  my $book  = ReadData ("test.xls");
  my $book  = ReadData ("test.xlsx");
  my $book  = ReadData ("test.xlsm");
+ my $book  = ReadData ("test.gnumeric");
  my $book  = ReadData ($fh, parser => "xls");
 
  Spreadsheet::Read::add ($book, "sheet.csv");
@@ -33,11 +34,11 @@ package Spreadsheet::Read;
 
 =cut
 
-use 5.8.1;
+use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = "0.84";
+our $VERSION = "0.87";
 sub  Version { $VERSION }
 
 use Carp;
@@ -51,27 +52,28 @@ use File::Temp   qw( );
 use Data::Dumper;
 
 my @parsers = (
-    [ csv  => "Text::CSV_XS",				"0.71"		],
-    [ csv  => "Text::CSV_PP",				"1.17"		],
-    [ csv  => "Text::CSV",				"1.17"		],
-    [ ods  => "Spreadsheet::ParseODS",			"0.26"		],
-    [ ods  => "Spreadsheet::ReadSXC",			"0.26"		],
-    [ sxc  => "Spreadsheet::ParseODS",			"0.26"		],
-    [ sxc  => "Spreadsheet::ReadSXC",			"0.26"		],
-    [ xls  => "Spreadsheet::ParseExcel",		"0.34"		],
-    [ xlsx => "Spreadsheet::ParseXLSX",			"0.24"		],
-    [ xlsm => "Spreadsheet::ParseXLSX",			"0.24"		],
-    [ xlsx => "Spreadsheet::XLSX",			"0.13"		],
-#   [ prl  => "Spreadsheet::Perl",			""		],
-    [ sc   => "Spreadsheet::Read",			"0.01"		],
+    [ csv	=> "Text::CSV_XS",			"0.71"		],
+    [ csv	=> "Text::CSV_PP",			"1.17"		],
+    [ csv	=> "Text::CSV",				"1.17"		],
+    [ ods	=> "Spreadsheet::ParseODS",		"0.26"		],
+    [ ods	=> "Spreadsheet::ReadSXC",		"0.26"		],
+    [ sxc	=> "Spreadsheet::ParseODS",		"0.26"		],
+    [ sxc	=> "Spreadsheet::ReadSXC",		"0.26"		],
+    [ xls	=> "Spreadsheet::ParseExcel",		"0.34"		],
+    [ xlsx	=> "Spreadsheet::ParseXLSX",		"0.24"		],
+    [ xlsm	=> "Spreadsheet::ParseXLSX",		"0.24"		],
+    [ xlsx	=> "Spreadsheet::XLSX",			"0.13"		],
+#   [ prl	=> "Spreadsheet::Perl",			""		],
+    [ sc	=> "Spreadsheet::Read",			"0.01"		],
+    [ gnumeric	=> "Spreadsheet::ReadGnumeric",		"0.2"		],
 
-    [ zzz1 => "Z10::Just::For::Testing",		"1.23"		],
-    [ zzz2 => "Z20::Just::For::Testing",		""		],
-    [ zzz3 => "Z30::Just::For::Testing",		"1.00"		],
+    [ zzz1	=> "Z10::Just::For::Testing",		"1.23"		],
+    [ zzz2	=> "Z20::Just::For::Testing",		""		],
+    [ zzz3	=> "Z30::Just::For::Testing",		"1.00"		],
 
     # Helper modules
-    [ ios  => "IO::Scalar",				""		],
-    [ dmp  => "Data::Peek",				""		],
+    [ ios	=> "IO::Scalar",				""		],
+    [ dmp	=> "Data::Peek",				""		],
     );
 my %can = ( supports => { map { $_->[1] => $_->[2] } @parsers });
 foreach my $p (@parsers) {
@@ -246,8 +248,10 @@ sub parses {
 
 sub sheets {
     my $ctrl = shift->[0];
-    my %s = %{$ctrl->{sheet}};
-    wantarray ? sort { $s{$a} <=> $s{$b} } keys %s : $ctrl->{sheets};
+    wantarray or return $ctrl->{sheets};
+
+    my $s = $ctrl->{sheet} or return (); # No labels defined
+    sort { $s->{$a} <=> $s->{$b} } keys %$s;
     } # sheets
 
 # col2label (4) => "D"
@@ -342,7 +346,10 @@ sub sheet {
 sub _clipsheets {
     my ($opt, $ref) = @_;
 
-    $ref->[0]{sheets} or return $ref;
+    unless ($ref->[0]{sheets}) {
+	$ref->{sheet} ||= {};
+	return $ref;
+	}
 
     my ($rc, $cl)      = ($opt->{rc},   $opt->{cells});
     my ($oc, $os, $oa) = ($opt->{clip}, $opt->{strip}, $opt->{attr});
@@ -445,6 +452,43 @@ sub _xls_fill {
     return _xls_color ($bg);
     } # _xls_fill
 
+sub _missing_parser {
+    my ($type, $suggest) = (shift, "");
+    for (@parsers) {
+	$_->[0] eq lc $type or next;
+	$suggest = "\nPlease install $_->[1]";
+	}
+    "No parser for $type found$suggest\n";
+    } # _missing_parser
+
+sub _txt_is_xml {
+    # Return true if $txt is gzipped or contains XML.  If we are also passed
+    # $ns_uri_of_interest, $txt must contain it in the first 1000 or so
+    # characters (but we try to search quickly rather than precisely).
+    my ($txt, $ns_uri_of_interest) = @_;
+
+    ref $txt and return; # Can't tell (unless we assume the stream is seekable).
+
+    if ($txt =~ m/\A\037\213/) {
+	# Literal gzipped string (/usr/share/misc/magic).  [this is a hack that
+	# works only because Gnumeric is the only format that uses gzip.  --
+	# rgr, 13-Jan-2023]
+	return 1;
+	}
+
+    if ($txt =~ m/\A<\?xml/) {
+	$ns_uri_of_interest or return 1;
+	$ns_uri_of_interest =~ s/([^\w\d])/\\$1/g;
+	my $prefix = length ($txt) > 10000 ? substr $txt, 0, 1000 : $txt;
+	return $prefix =~ m/xmlns:\w+=.$ns_uri_of_interest/;
+	}
+
+    $txt =~ m/\n/		and return; # safeguard for older perl versions
+    open my $in, "<", $txt	or  return;
+    read $in, my $block, 1000	or  return;
+    return _txt_is_xml ($block, $ns_uri_of_interest);
+    } # _txt_is_xml
+
 sub ReadData {
     my $txt = shift	or  return;
 
@@ -485,7 +529,7 @@ sub ReadData {
     $io_ref &&  eof $txt and do { $@ = "Empty stream";  return };
 
     if ($opt{parser} ? $_parser eq "csv" : ($io_fil && $txt =~ m/\.(csv)$/i)) {
-	$can{csv} or croak "CSV parser not installed";
+	$can{csv} or croak _missing_parser ("CSV");
 
 	my $label = defined $opt{label} ? $opt{label} : $io_fil ? $txt : "IO";
 
@@ -601,7 +645,7 @@ sub ReadData {
 	     || $txt =~ m{\A.{2080}Microsoft Excel 5.0 Worksheet}
 	     || $txt =~ m{\A\x09\x04\x06\x00\x00\x00\x10\x00}
 		) {
-	    $can{xls} or croak "Spreadsheet::ParseExcel not installed";
+	    $can{xls} or croak _missing_parser ("XLS");
 	    my $tmpfile;
 	    if ($can{ios}) { # Do not use a temp file if IO::Scalar is available
 		$tmpfile = \$txt;
@@ -619,7 +663,7 @@ sub ReadData {
 	elsif ( # /usr/share/misc/magic
 		$txt =~ m{\APK\003\004.{4,30}(?:\[Content_Types\]\.xml|_rels/\.rels)}
 		) {
-	    $can{xlsx} or croak "XLSX parser not installed";
+	    $can{xlsx} or croak _missing_parser ("XLSX");
 	    my $tmpfile;
 	    if ($can{ios}) { # Do not use a temp file if IO::Scalar is available
 		$tmpfile = \$txt;
@@ -637,7 +681,7 @@ sub ReadData {
 	elsif ( # /usr/share/misc/magic
 		$txt =~ m{\APK\003\004.{9,30}\Qmimetypeapplication/vnd.oasis.opendocument.spreadsheet}
 		) {
-	    $can{ods} or croak "ODS parser not installed";
+	    $can{ods} or croak _missing_parser ("ODS");
 	    my $tmpfile;
 	    if ($can{ios}) { # Do not use a temp file if IO::Scalar is available
 		$tmpfile = \$txt;
@@ -661,8 +705,7 @@ sub ReadData {
 		     : ($io_fil && $txt =~ m/\.(xls[xm]?)$/i &&
 		      ($_parser = _parser ($1)))) {
 	my $parse_type = $_parser =~ m/x$/i  ? "XLSX" : "XLS";
-	my $parser = $can{lc $parse_type} or
-	    croak "Parser for $parse_type is not installed";
+	my $parser = $can{lc $parse_type} or croak _missing_parser ($parse_type);
 	#$debug and print STDERR __FILE__, "#", __LINE__, " | $_parser | $parser | $parse_type\n";
 	$debug and print STDERR "Opening $parse_type ", $io_ref ? "<REF>" : $txt,
 	    " using $parser-", $can{lc $parse_type}->VERSION, "\n";
@@ -970,7 +1013,7 @@ sub ReadData {
 	     and ($can{$_parser} || "") !~ m/sxc/i) {
 	my $parse_type = "ODS";
 	my $parser = $can{lc $parse_type} or
-	    croak "Parser for $parse_type is not installed";
+	    croak _missing_parser ($parse_type);
 	#$debug and print STDERR __FILE__, "#", __LINE__, " | $_parser | $parser | $parse_type\n";
 	$debug and print STDERR "Opening $parse_type ", $io_ref ? "<REF>" : $txt,
 	    " using $parser-", $can{lc $parse_type}->VERSION, "\n";
@@ -1196,9 +1239,22 @@ sub ReadData {
 	return _clipsheets \%opt, [ @data ];
 	}
 
+    if ($opt{parser} ? _parser ($opt{parser}) eq "gnumeric"
+		     : _txt_is_xml ($txt, "http://www.gnumeric.org/v10.dtd")) {
+	$can{gnumeric} or croak _missing_parser ("gnumeric");
+
+	my $gnm = $can{gnumeric}->new (%parser_opts,
+				       attr      => $opt{attr},
+				       cells     => $opt{cells},
+				       merge     => $opt{merge},
+				       rc        => $opt{rc},
+				       gzipped_p => $opt{gzipped_p});
+	return _clipsheets \%opt, $gnm->parse ($txt);
+	}
+
     if ($opt{parser} ? _parser ($opt{parser}) eq "sxc"
 		     : ($txt =~ m/^<\?xml/ or -f $txt)) {
-	$can{sxc} or croak "Spreadsheet::ReadSXC not installed";
+	$can{sxc} or croak _missing_parser ("SXC");
 
 	ref $txt && $can{sxc}->VERSION <= 0.23 and
 	    croak ("Sorry, references as input are not supported by Spreadsheet::ReadSXC before 0.23");
@@ -1206,7 +1262,7 @@ sub ReadData {
 	my $using = "using $can{sxc}-" . $can{sxc}->VERSION;
 	my $sxc_options = { %parser_opts, OrderBySheet => 1 }; # New interface 0.20 and up
 	my $sxc;
-	   if ($txt =~ m/\.(sxc|ods)$/i) {
+	if ($txt =~ m/\.(sxc|ods)$/i) {
 	    $debug and print STDERR "Opening \U$1\E $txt $using\n";
 	    $debug and print STDERR __FILE__, "#", __LINE__, "\n";
 	    $sxc = Spreadsheet::ReadSXC::read_sxc      ($txt, $sxc_options) or return;
@@ -1498,7 +1554,7 @@ L<Spreadsheet::ReadSXC|https://metacpan.org/release/Spreadsheet-ReadSXC>
 For Microsoft Excel this module uses
 L<Spreadsheet::ParseExcel|https://metacpan.org/release/Spreadsheet-ParseExcel>,
 L<Spreadsheet::ParseXLSX|https://metacpan.org/release/Spreadsheet-ParseXLSX>, or
-L<Spreadsheet::XLSX|https://metacpan.org/release/Spreadsheet-XLSX> (stronly
+L<Spreadsheet::XLSX|https://metacpan.org/release/Spreadsheet-XLSX> (strongly
 discouraged).
 
 For CSV this module uses L<Text::CSV_XS|https://metacpan.org/release/Text-CSV_XS>
@@ -1925,7 +1981,11 @@ returns and is only kept for backward compatibility reasons.
  my @sheets = $book->sheets; # OO
 
 In scalar context return the number of sheets in the book.
-In list context return the labels of the sheets in the book.
+
+In list context return the labels of the sheets in the book. This list only
+returns known unique labels in sorted order. Sheets could have no label or
+there can be more sheets with the same label (depends on the spreadsheet
+format and the parser used).
 
 =head3 sheet
 
@@ -2450,7 +2510,7 @@ Convert a spreadsheet to CSV. This is just a small wrapper over C<xlscat>.
      -Z    --zip     Convert sheets to CSV's in ZIP
      -J s  --join=s  Use s to join filename-sheetname (-)
      -o f  --out=f   Set output filename
-     -i f  --in=f    Set infut  filename
+     -i f  --in=f    Set input  filename
      -f    --force   Force overwrite output if exists
      -s s  --sep=s   Set CSV separator character
  Unless -A is used, all other options are passed on to xlscat
@@ -2525,12 +2585,6 @@ Under investigation:
 
 =over 2
 
-=item Gnumeric (.gnumeric)
-
-I have seen no existing CPAN module yet.
-
-It is gzip'ed XML
-
 =item Kspread (.ksp)
 
 Now knows as Calligra Sheets.
@@ -2597,6 +2651,12 @@ successor of  L<Spreadsheet::ReadSXC|https://metacpan.org/release/Spreadsheet-Re
 L<Spreadsheet::ReadSXC|https://metacpan.org/release/Spreadsheet-ReadSXC> is a
 parser for OpenOffice/LibreOffice (.sxc and .ods) spreadsheet files.
 
+=item Spreadsheet::ReadGnumeric
+
+L<Spreadsheet::ReadGnumeric|https://metacpan.org/release/Spreadsheet-ReadGnumeric>
+is a parser for L<Gnumeric|http://www.gnumeric.org/> (.gnumeric) spreadsheet
+files.
+
 =item Spreadsheet::BasicRead
 
 See L<Spreadsheet::BasicRead|https://metacpan.org/release/Spreadsheet-BasicRead>
@@ -2635,7 +2695,7 @@ H.Merijn Brand <perl5@tux.freedom.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2021 H.Merijn Brand
+Copyright (C) 2005-2023 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
